#include "stdafx.h"

#include "common.h"


namespace cmmn {

	//
	//			omega_a = sqr(sum_lambda) / var(Y):						 	 (6.20a)
	//			omega_b = sqr(sum_lambda) / (sqr(sum_lambda) + sum(psi**2)): (6.20b)
	//
	void CalcOmega_ab( double & omega_a, double & omega_b, double ** sjk, int M, double * Lambda, double & GFI, double * psi2, System::IO::StreamWriter ^ fout ){
		myMat aa( sjk, M, M );
		myMat EigenVec(M, M);
		myMat EigenVal(M, 1);
		int n_eigen;

		QL_decom ql;

		int ck =  ql.DoQL( aa, EigenVec, EigenVal, n_eigen );
		System::Diagnostics::Debug::WriteLine("@1...ck = " + ck.ToString());
		if (ck != 0){
			throw gcnew System::Exception("Error @CalcOmega");
		}

		for (int i = 0; i < M; i++){
			Lambda[i] = EigenVec[i][0] * Math::Sqrt(EigenVal[0][0]);
		}

		double * lambda0 = new double[M];

		double ck1;
		do{
			for (int i = 0; i < M; i++)
				for (int j = 0; j < M; j++)
					if (i == j){
							lambda0[i] = Lambda[i];
							aa[i][i] = sqr(Lambda[i]);
					}else{
							aa[i][j] = sjk[i][j];
					}

			System::Diagnostics::Debug::WriteLine("Before DoQL");
			
			QL_decom ql;
			ck =  ql.DoQL( aa, EigenVec, EigenVal, n_eigen );
			System::Diagnostics::Debug::WriteLine("@do-while...ck = " + ck.ToString());
			if (ck != 0){
				throw gcnew System::Exception("Error @CalcOmega");
			}

			ck1 = 0.0;
			double ck_v = 0.0;
			for (int i = 0; i < M; i++){
				Lambda[i] = EigenVec[i][0] * Math::Sqrt(EigenVal[0][0]);
				ck1 += Math::Abs(Lambda[i] - lambda0[i]);
				if (ck_v < Math::Abs(Lambda[i]))
						ck_v = Math::Abs(Lambda[i]);
			}
			ck1 /= ck_v;
		}while(ck1 > 1.0e-12);

		delete [] lambda0;

		double sum_Lambda = 0.0;
		for (int i = 0; i < M; i++)
			sum_Lambda += Lambda[i];
		double Sgm2T = sqr(sum_Lambda);
		double Sgm2Y = 0.0;
		for (int i = 0; i < M; i++)
			for (int j = 0; j < M; j++)
				Sgm2Y += sjk[i][j];

		omega_a = Sgm2T / Sgm2Y;

		for (int i = 0; i < M; i++)
			psi2[i] = sjk[i][i] - sqr(Lambda[i]);

		fout->WriteLine();
		for (int i = 0; i < M; i++)
			fout->WriteLine("Lambda[" + i.ToString() + "]**2 / VarX[" + i.ToString() + "]**2 = " 
			+ (sqr(Lambda[i]) / sjk[i][i]).ToString("F5"));

		double sum_psi2 = 0.0;
		for (int i = 0; i < M; i++)
			sum_psi2 += psi2[i];
		
		omega_b = Sgm2T / (Sgm2T + sum_psi2);

		double sum_d2 = 0.0, sum_Td2 = 0.0;
		for (int i = 0; i < M; i++)
			for (int j = 0; j < M; j++){
				sum_Td2 += sqr(sjk[i][j]);
				if (i != j)	sum_d2 += sqr(sjk[i][j] - Lambda[i] * Lambda[j]);
			}
		GFI = 1.0 - sum_d2 / sum_Td2;

		return;
	}

}
