#include "stdafx.h"
#include "mcmc.h"
#include "rnkissM.h"
#include "DispMessage.h"

using namespace MyRNKISS;
using namespace OmegaMCMC;

namespace ns_mcmc{

	double ** param_mu, ** param_lmbd, ** param_sgmE;
	double * omega;
	 double * omegaDistri;

	double MCMC::LogLTi(int i, double Ti, double *mu, double *lmbd, double *sgmE){
		double v = -0.5 * sqr( Ti );
		for (int j = 0; j < M; j++){
			v += -Math::Log( sgmE[j] ) - 0.5 * sqr( (X[j][i] - mu[j] - lmbd[j] * Ti) / sgmE[j] );
		}
		return v;
	}

	double MCMC::LogLj( int j, double * T, double muj, double lmbdj, double sgmEj ){
		double v = -NCases * Math::Log( sgmEj );
		for (int i = 0; i < NCases; i++){
			v += -0.5 * sqr( (X[j][i] - muj - lmbdj * T[i]) / sgmEj );
		}
		return v;
	}

	double myExp( double v ){
			double tv = 2.0;
			if (v < 1.0){
				tv = Math::Exp( v );
			}
			return tv;
	}

	void MCMC::Init( int vnmcmc, int vID, int vM, int vNCases, int vNSimu, double ** vX ){
		rn = new myrn();
		rn->Init( vnmcmc, vID );
		ID = vID;

		M = vM;
		NCases = vNCases;
		NSimu = vNSimu;

		mu = new double*[NSimu+1];
		for (int i = 0; i < NSimu+1; i++)
			mu[i] = new double[M];
		lmbd = new double*[NSimu+1];
		for (int i = 0; i < NSimu+1; i++)
			lmbd[i] = new double[M];
		T = new double*[NSimu+1];
		for (int i = 0; i < NSimu+1; i++)
			T[i] = new double[NCases];
		sgmE = new double*[NSimu+1];
		for (int i = 0; i < NSimu+1; i++)
			sgmE[i] = new double[M];
		X = new double*[M];
		for (int j = 0; j < M; j++){
				X[j] = new double[NCases];
				for (int i = 0; i < NCases; i++)
					X[j][i] = vX[j][i];
		}

		acpt_mu = new int[M];
		for (int i = 0; i < M; i++)
			acpt_mu[i] = 0;
		acpt_lmbd = new int[M];
		for (int i = 0; i < M; i++)
			acpt_lmbd[i] = 0;
		acpt_T = new int[NCases];
		for (int i = 0; i < NCases; i++)
			acpt_T[i] = 0;
		acpt_sgmE = new int[M];
		for (int i = 0; i < M; i++)
			acpt_sgmE[i] = 0;
		sgmMu = new double[M];
		sgmLmbd = new double[M];
		sgmT = new double[NCases];
		sgm_sgmE = new double[M];

		return;
	}

	void MCMC::clean(){
		if (rn != nullptr) delete rn;

		if (mu != nullptr){
			for (int i = 0; i < NSimu+1; i++)
				delete [] mu[i];
			delete [] mu;
		}
		if (lmbd != nullptr){
			for (int i = 0; i < NSimu+1; i++)
				delete [] lmbd[i];
			delete [] lmbd;
		}
		if (T != nullptr){
			for (int i = 0; i < NSimu+1; i++)
				delete [] T[i];
			delete [] T;
		}
		if (sgmE != nullptr){
			for (int i = 0; i < NSimu+1; i++)
				delete [] sgmE[i];
		}
		if (X != nullptr){
			for (int i = 0; i < M; i++)
				delete [] X[i];
			delete [] X;
		}
		if (acpt_mu != nullptr)
			delete [] acpt_mu;
		if (acpt_lmbd != nullptr)
			delete [] acpt_lmbd;
		if (acpt_T != nullptr)
			delete [] acpt_T;
		if (acpt_sgmE != nullptr)
			delete [] acpt_sgmE;
		if (sgmMu != nullptr)
			delete [] sgmMu;
		if (sgmLmbd != nullptr)
			delete [] sgmLmbd;
		if (sgmT != nullptr)
			delete [] sgmT;
		if (sgm_sgmE != nullptr)
			delete [] sgm_sgmE;

		return;
	}

	void MCMC::do_mcmc(){
		DispMessage ^ cf = gcnew DispMessage();
		cf->Show();
		cf->ShowMessage(msg + "mcmc-" + ID.ToString() + " started.");

		for (int isimu = 0; isimu < NSimu; isimu++){
			if (isimu % 100 == 0){
				cf->ShowMessage(msg + "mcmc-" + ID.ToString() + "   " + isimu.ToString() + "/" + NSimu.ToString());
			}

			for (int i = 0; i < NCases; i++){
				double y = rn->normalMS( T[isimu][i], sgmT[i] );
				double vnum = LogLTi( i, y, mu[isimu], lmbd[isimu], sgmE[isimu] );
				double vden = LogLTi( i, T[isimu][i], mu[isimu], lmbd[isimu], sgmE[isimu] );
				double a = myExp( vnum - vden );
				if (rn->uni() < a){
						T[isimu+1][i] = y;
						acpt_T[i]++;
				}else{
						T[isimu+1][i] = T[isimu][i];
				}
			}
			for (int j = 0; j < M; j++){
				double y = rn->normalMS( mu[isimu][j], sgmMu[j] );
				double vnum = LogLj( j, T[isimu+1], y, lmbd[isimu][j], sgmE[isimu][j] );
				double vden = LogLj( j, T[isimu+1], mu[isimu][j], lmbd[isimu][j], sgmE[isimu][j] );
				double a = myExp( vnum - vden );
				if (rn->uni() < a){
						mu[isimu+1][j] = y;
						acpt_mu[j]++;
				}else{
						mu[isimu+1][j] = mu[isimu][j];
				}
			}
			for (int j = 0; j < M; j++){
				double y = rn->normalMS( lmbd[isimu][j], sgmLmbd[j] );
				double a;
				if (y <= 0.0){
						a = -1.0;
				}else{
						double vnum = LogLj( j, T[isimu+1], mu[isimu+1][j], y, sgmE[isimu][j] );
						double vden = LogLj( j, T[isimu+1], mu[isimu+1][j], lmbd[isimu][j], sgmE[isimu][j] );
						a = myExp( vnum - vden );
				}
				if (rn->uni() < a){
						lmbd[isimu+1][j] = y;
						acpt_lmbd[j]++;
				}else{
						lmbd[isimu+1][j] = lmbd[isimu][j];
				}
			}
			for (int j = 0; j < M; j++){
				double y = rn->normalMS( sgmE[isimu][j], sgm_sgmE[j] );
				double a;
				if (y <= 0.0){
						a = -1.0;
				}else{
						double vnum = LogLj( j, T[isimu+1], mu[isimu+1][j], lmbd[isimu+1][j], y );
						double vden = LogLj( j, T[isimu+1], mu[isimu+1][j], lmbd[isimu+1][j], sgmE[isimu][j] );
						a = myExp( vnum - vden );
				}
				if (rn->uni() < a){
						sgmE[isimu+1][j] = y;
						acpt_sgmE[j]++;
				}else{
						sgmE[isimu+1][j] = sgmE[isimu][j];
				}
			}
		}	//  The end of for-isimu.

		cf->Close();
		return;
	}




	//
	//								Quick Sort
	//
	//      S. Kawai (1988).  Methods of programming (in Japanese). Tokyo: Iwanami-shoten.
	//		S. Ishihata (1989). Algorithms and Data Structure (in Japanese). Tokyo:Iwanami-shoten
	//
	void qSort( double * d, int Lb, int Ub ){
		if (Lb < Ub){
			int p = (Lb + Ub) / 2;
			double c = d[p];
			int i = Lb;
			int j = Ub;
			do {
				while (d[i] < c) i++;
				while (c < d[j]) j--;
				if (i <= j){
					double v = d[i];
					d[i] = d[j];
					d[j] = v;
					i++;
					j--;
				}
			} while (i <= j);
			qSort(d, Lb, j);
			qSort(d, i, Ub);
		}
		return;
	}

	void calc_CI( double * d, int NSimu, double & CIL, double & Med, double & CIR ){
		Med = d[NSimu/2];
		double CIW = d[NSimu-1] - d[0];
		int posL, posR;
		for (int i = 0; i < int(NSimu*0.05); i++){
			if ((d[int(0.95*NSimu) + i] - d[i]) < CIW){
				CIW = d[int(0.95*NSimu) + i] - d[i];
				posR = int(0.95*NSimu) + i;
				posL = i;
			}
		}
		CIL = d[posL];
		CIR = d[posR];
		return;
	}

	void CalcHPDIntvl( double * d, int n, double p, double & LB, double & UB ){
		int sp = 0, ep = int(n * p);
		double vintvl = d[ep] - d[sp];
		int ckL = sp, ckU = ep;
		while (ckU < n){
			if ((d[ckU] - d[ckL]) < vintvl){
					sp = ckL; 
					ep = ckU;
					vintvl = d[ckU] - d[ckL];
			}
			ckU++;
			ckL++;
		}
		LB = d[sp];
		UB = d[ep];
	}




}
