#ifndef MCMC_CK
#define MCMC_CK

#include "rnkissM.h"
#include "DispMessage.h"

using namespace MyRNKISS;
using namespace OmegaMCMC;

namespace ns_mcmc{

	extern double ** param_mu, ** param_lmbd, ** param_sgmE;
	extern double * omega;
	extern double * omegaDistri;

	ref class MCMC {
		public:
				int ID;
				myrn * rn;
				String ^ msg;
				int NSimu, M, NCases;
				double ** mu, ** lmbd, ** T, ** sgmE, ** X;
				int * acpt_mu, * acpt_lmbd, * acpt_T, * acpt_sgmE;
				double * sgmMu, * sgmLmbd, * sgmT, * sgm_sgmE;

				MCMC(){ msg = ""; }
				
				void clean();

				~MCMC(){ clean(); }
				!MCMC(){ clean(); }

				double sqr( double v ){
						return v * v;
				}

				double LogLTi( int i, double Ti, double * mu, double * lmbd, double * sgmE);
				double LogLj( int j, double * T, double muj, double lmbdj, double sgmEj );

				void Init( int vnmcmc, int vID, int vM, int vNCases, int vNSimu, double ** vX );

				void do_mcmc();
	};


	void qSort( double * d, int Lb, int Ub );

	void calc_CI( double * d, int NSimu, double & CIL, double & Med, double & CIR );

	void CalcHPDIntvl( double * d, int n, double p, double & LB, double & UB );


	ref class CalcDistri {
			public:
						double * distri;
						int n, n1, n2;
						double s_x, e_x, w;

						double * samples;
						int n_data;

						String ^ msg;

						void clearMe(){
							if (samples != nullptr) delete [] samples;
						}

						~CalcDistri(){ clearMe(); }
						!CalcDistri(){ clearMe(); }

						void Init( int vn, int vn1, int vn2, double vs_x, double ve_x, double vw, double * vsamples, int vn_data, String ^ vmsg ){
								n = vn;
								distri = new double[n+1];
								n1 = vn1;
								n2 = vn2;
								s_x = vs_x;
								e_x = ve_x;
								w = vw;
								n_data = vn_data;
								samples = new double[n_data];
								for (int i = 0; i < n_data; i++)
									samples[i] = vsamples[i];
								msg = vmsg;
						}

						inline double sqr( double v ){
								return  v * v;
						}


						double kernel( double x, double c, double w ){
							return Math::Exp(-0.5 * sqr((x - c) / w));
						}


						double k_ave( double x ){
							double v = 0.0;
							for (int i = 0; i < n_data; i++)
								v += kernel( x, samples[i], w );
							v /= Math::Sqrt( 2.0 * Math::PI ) * w * n_data;

							return v;
						}

						void do_calc(){
								DispMessage ^ cf = gcnew DispMessage();
								cf->Show();
								cf->ShowMessage( msg + "I'm calculating..." );

								for (int i = n1; i <= n2; i++){
									if ((i % 1000) == 0)
										cf->ShowMessage( msg + ": " + i.ToString() + "/" + (n2 - n1).ToString() );

									distri[i] = k_ave( s_x + (e_x - s_x) * i / double(n) );
								}

								cf->Close();
						}

	};

}

#endif
